"""
Components are elements that define a link between geometrical dimensions\
and electrical behavior using physical parameter.
For example, Inductor create a link between the inner and outer diameter and
its self-inductance.
"""
